==> pan-0.6.6.tar.bz2: pan-0.6.6/pan/message-send.c <==
/* Pan - Message sending over NNTP and SMTP
 *
 * Copyright (C) 1999 Jason Leach
 * Copyright (C) 1999 Matt Eagleson (e.Messiah@superpimp.org)
 *
 * Authors: Matt Eagleson <e.Messiah@superpimp.org>
 *          Jason Leach <leach@wam.umd.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef __FreeBSD__
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>
#endif

#include <arpa/inet.h>
#include <netdb.h>
#include <pwd.h>
#include "pan.h"
#include "../config.h"

#ifdef AF_NEEDS_SYS_SOCKET_H
#include <sys/socket.h>
#endif


static gchar* get_host_name (void);
static gchar* get_fqdn (gchar *host);
static gchar* generate_references (Message *msg);


static gint
check_from_syntax (gchar *s)
{
	return 1;
}


static gint
check_newsgroups_syntax (gchar *s)
{
	return 1;
}

static gchar*
check_body (const gchar* body)
{
	if ( !body || !*body )
		return g_strdup ( _( "Empty Message" ) );

	/* FIXME: check number of new lines compared to number of quoted lines */
	return NULL;
}

static gchar *
generate_references (Message *msg)
{
#define GNKSA_LENGTH 998
	gchar buf[GNKSA_LENGTH + 1];
	gchar *p = NULL;
	gchar *refs = NULL;
	gchar *new_refs = NULL;
	gchar *pch = NULL;
	unsigned int total_len, len;

//	p = extract_header (mw->article, "References");
//	if (p)
//		refs = g_strconcat (p, " ", extract_header(mw->article, "Message-ID"), NULL);
//	else
//		refs = g_strconcat (extract_header(mw->article, "Message-ID"), NULL);
//	g_free (p);
	
	if (msg->references) {
		refs = g_strconcat (msg->references, " ", Pan.active_message_id, NULL);
	}
	else if (Pan.active_message_id) {
		refs = g_strconcat (Pan.active_message_id, NULL);
	}
	else {
		refs = g_strdup ( "" );
	}



	len = strlen (refs);
	if (len <= GNKSA_LENGTH) {
		return refs;
	}

	/* The reference header is too long so start truncating */
	/* We have to preserve the first message-id */
	p = strchr(refs, '>');
	if (p == NULL)
		return NULL;
	p++;
	len = p - refs;
	if (len >= GNKSA_LENGTH) {
		return refs;
	}
	strncpy(buf, refs, len);
	buf[len] = 0;
	new_refs = g_strconcat(buf, " ", NULL);
	total_len = len + 1;

	refs += len;
	if ((*refs == '\t') || (*refs == ' '))
		refs++;					/* already handled space */

	/* Now use message-ids at end of the line */
	p = refs + strlen(refs);

	while (1) {
		if (*p == '<') {
			if (strlen(p) + total_len > GNKSA_LENGTH) {
				p = strchr(p + 1, '<');
				if (p == NULL)
					p = refs;
				break;
			}
		}
		if (p == refs)
			break;
		p--;
	}

	pch = new_refs;
	new_refs = g_strconcat (new_refs, p, NULL);
	g_free ( pch );
	g_free ( refs );
	return new_refs;
}


static gchar*
generate_message_id (void)
{
	unsigned long now;
	unsigned long randy;
	static unsigned char baseid[64];
	gchar *b, *t, tmp[32];
	char *chars32 = "0123456789abcdefghijklmnopqrstuv";
	static unsigned long last_now;

	while (1) {
		time((time_t *) & now);
		if (now != last_now)
			break;
		sleep (1);
	}
	last_now = now;
	srand (now + getpid());
	randy = (unsigned long) rand();

	b = baseid;
	t = tmp;
	while (now) {
		*t++ = chars32[now & 0x1F];
		now = now >> 5;
	}

	while (t > tmp) {
		t--;
		*b++ = *t;
	}
	*b++ = '.';

	t = tmp;
	while (randy) {
		*t++ = chars32[randy & 0x1F];
		randy = randy >> 5;
	}
	while (t > tmp) {
		t--;
		*b++ = *t;
	}

	*b++ = '@';
	return (char *) baseid;
}


/*--------------------------------------------------------------------
 * FIXME: move into util.c (or make me static?)
 *--------------------------------------------------------------------*/
static gchar *
get_host_name (void)
{
	gchar *ptr;
	gchar hostname[MAXHOSTNAMELEN + 1];

	hostname[0] = '\0';

	if ((gethostname(hostname, sizeof(hostname))) == -1) {
		if ((ptr = getenv("HOST")) != NULL)
			strncpy(hostname, ptr, MAXHOSTNAMELEN);
		else if ((ptr = getenv("HOSTNAME")) != NULL)
			strncpy(hostname, ptr, MAXHOSTNAMELEN);
		else
			hostname[0] = '\0';
	}
	hostname[MAXHOSTNAMELEN] = '\0';
	ptr = strtok (hostname, ".");
	return g_strdup ( ptr );
}


/*--------------------------------------------------------------------
 * get the fully qualified domain name
 *--------------------------------------------------------------------*/
static gchar *
get_fqdn (gchar *host)
{
	char name[MAXHOSTNAMELEN + 2];
	char line[1025];
	char *cp, *domain;
	static char fqdn[1024];
	struct hostent *hp;
	struct in_addr in;
	FILE *inf;

	*fqdn = '\0';
	domain = NULL;

	name[MAXHOSTNAMELEN] = '\0';
	if (host) {
		if (strchr(host, '.'))
			return host;
		strncpy(name, host, MAXHOSTNAMELEN);
	}
	else {
		if (gethostname(name, MAXHOSTNAMELEN))
			return NULL;
	}

	if ('0' <= *name && *name <= '9') {
		in.s_addr = inet_addr(name);
		if ((hp = gethostbyaddr((char *) &in.s_addr, 4, AF_INET)))
			in.s_addr = (*hp->h_addr);
		return (hp && strchr(hp->h_name, '.') ? hp->h_name : (gchar *) inet_ntoa(in));
	}
	if ((hp = gethostbyname(name)) && !strchr(hp->h_name, '.'))
		if ((hp = gethostbyaddr(hp->h_addr, hp->h_length, hp->h_addrtype)))
			in.s_addr = (*hp->h_addr);

	sprintf(fqdn, "%s", hp ? strchr(hp->h_name, '.')
		? hp->h_name : (gchar *) inet_ntoa(in)
		: "");
	if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) {
		*fqdn = '\0';
		inf = fopen("/etc/resolv.conf", "r");
		if (inf) {
			while (fgets(line, 1024, inf)) {
				line[1024] = '\0';
				(void) Pan_strtrim(line);
				if (strncmp(line, "domain ", 7) == 0) {
					domain = line + 7;
					break;
				}
				if (strncmp(line, "search ", 7) == 0) {
					domain = line + 7;
					cp = strchr(domain, ' ');
					if (cp)
						*cp = '\0';
					break;
				}
			}
			if (domain)
				sprintf(fqdn, "%s.%s", name, domain);
			fclose(inf);
		}
	}
	return (fqdn);
}

static gchar *
reformat_body (gchar *body)
{
	gchar *tmp;
	gchar *body2 = g_malloc0 (sizeof (body));

	while ((tmp = strtok (body, "\n"))) {
		tmp = wrap_long_lines (tmp, 78);
		g_strconcat (body2, tmp, NULL);
	}

	return body2;
}

void
message_send (GtkWidget *widget, Message *msg)
{
	gchar *subject, *from, *newsgroups, *message, *path, *organization;
	gchar *tmp;
	gchar *buf;
	static char *host = NULL;
	static char *uname = NULL;
	static char *fqdn = NULL;
	gchar *body;
	gchar *err;

	if (msg->window == NULL)
		g_error ("WTF!! msg->window == NULL\n");
	if (host == NULL)
		host = get_host_name ();
	if (fqdn == NULL)
		fqdn = get_fqdn (host);
	if (uname == NULL)
		uname = g_get_user_name ();

	path = g_strconcat (host, "!", uname, NULL);
	g_free ( host );

	from = gtk_editable_get_chars (GTK_EDITABLE (msg->window->from), 0, -1);
	newsgroups = gtk_editable_get_chars (GTK_EDITABLE (msg->window->newsgroups), 0, -1);
	subject = gtk_editable_get_chars (GTK_EDITABLE (msg->window->subject), 0, -1);
	organization = gtk_entry_get_text (GTK_ENTRY(msg->window->organization));

	err = check_body ( gtk_entry_get_text ( GTK_ENTRY(msg->window->body) ) );
	if ( err != NULL ) {
		pan_error_dialog ( _("Unable to post message:\n%s"), err );
		return;
	}

/* FIXME: doesn't do anything
	if (!check_from_syntax(from))
		g_error ("Bad From. Do something!");
	if (!check_newsgroups_syntax(newsgroups))
		g_error ("Bad Newsgroups. Do something!");
*/
	message = g_strconcat ("From: ", from, "\n",
			       "Path: ", path, "\n",
			       "Newsgroups: ", newsgroups, "\n",
			       "Subject: ", subject, "\n", NULL);
	g_free (path);
	g_free (from);
	g_free (newsgroups);
	g_free (subject);
	tmp = message;
	
	if (Pan.active_server->gen_msgid) {
		message = g_strconcat (tmp,"Message-ID: <", generate_message_id(), fqdn, ">\n", NULL);
		g_free (tmp);
		tmp = message;
	}
	if (msg) {
		gchar* pch = generate_references ( msg );
		message = g_strconcat (message, "References: ", pch, "\n", NULL );
		g_free ( pch );
		g_free ( tmp );
		tmp = message;
	}
	body = g_strconcat ("\n", gtk_editable_get_chars (GTK_EDITABLE(msg->window->body), 0, -1), NULL);
	buf = generate_date ();
	message = g_strconcat (tmp,
			       "Date: ", buf, "\n",
			       "Organization: ", organization, "\n",
			       "X-Newsreader: ", "Pan ", VERSION, "\n",
			       wrap_long_lines (body, 78), NULL);
	g_free (buf);
	g_free (tmp);
	g_free (body);

	queue_add_post ( message );
	g_free ( message );

	gtk_widget_destroy ( msg->window->window );
	message_free ( msg );
}


static GSList *
parse_to_field (MessageWindow *mw)
{
	return NULL;
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
static gint
determine_transport_type (const gchar *address)
{
	if ( strcasecmp (address, "@"))
		return EMAIL;
	else if ( strcasecmp (address, "."))
		return NNTP;
	else
		return 0;
}

==> pan-0.6.7.tar.bz2: pan-0.6.7/pan/message-send.c <==
/* Pan - Message sending over NNTP and SMTP
 *
 * Copyright (C) 1999 Jason Leach
 * Copyright (C) 1999 Matt Eagleson (e.Messiah@superpimp.org)
 *
 * Authors: Matt Eagleson <e.Messiah@superpimp.org>
 *          Jason Leach <leach@wam.umd.edu>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef __FreeBSD__
#include <unistd.h>
#include <netinet/in.h>
#include <sys/socket.h>
#endif

#include <arpa/inet.h>
#include <netdb.h>
#include <pwd.h>
#include "pan.h"
#include "../config.h"

#ifdef AF_NEEDS_SYS_SOCKET_H
#include <sys/socket.h>
#endif


static gchar* get_host_name (void);
static gchar* get_fqdn (gchar *host);
static gchar* generate_references (Message *msg);


static gint
check_from_syntax (gchar *s)
{
	return 1;
}


static gint
check_newsgroups_syntax (gchar *s)
{
	return 1;
}

static gchar*
check_body (const gchar* body)
{
	if ( !body || !*body )
		return g_strdup ( _( "Empty Message" ) );

	/* FIXME: check number of new lines compared to number of quoted lines */
	return NULL;
}

static gchar *
generate_references (Message *msg)
{
#define GNKSA_LENGTH 998
	gchar buf[GNKSA_LENGTH + 1];
	gchar *p = NULL;
	gchar *refs = NULL;
	gchar *new_refs = NULL;
	gchar *pch = NULL;
	unsigned int total_len, len;

//	p = extract_header (mw->article, "References");
//	if (p)
//		refs = g_strconcat (p, " ", extract_header(mw->article, "Message-ID"), NULL);
//	else
//		refs = g_strconcat (extract_header(mw->article, "Message-ID"), NULL);
//	g_free (p);
	
	if (msg->references) {
		refs = g_strconcat (msg->references, " ", Pan.active_message_id, NULL);
	}
	else if (Pan.active_message_id) {
		refs = g_strconcat (Pan.active_message_id, NULL);
	}
	else {
		refs = g_strdup ( "" );
	}



	len = strlen (refs);
	if (len <= GNKSA_LENGTH) {
		return refs;
	}

	/* The reference header is too long so start truncating */
	/* We have to preserve the first message-id */
	p = strchr(refs, '>');
	if (p == NULL)
		return NULL;
	p++;
	len = p - refs;
	if (len >= GNKSA_LENGTH) {
		return refs;
	}
	strncpy(buf, refs, len);
	buf[len] = 0;
	new_refs = g_strconcat(buf, " ", NULL);
	total_len = len + 1;

	refs += len;
	if ((*refs == '\t') || (*refs == ' '))
		refs++;					/* already handled space */

	/* Now use message-ids at end of the line */
	p = refs + strlen(refs);

	while (1) {
		if (*p == '<') {
			if (strlen(p) + total_len > GNKSA_LENGTH) {
				p = strchr(p + 1, '<');
				if (p == NULL)
					p = refs;
				break;
			}
		}
		if (p == refs)
			break;
		p--;
	}

	pch = new_refs;
	new_refs = g_strconcat (new_refs, p, NULL);
	g_free ( pch );
	g_free ( refs );
	return new_refs;
}


static gchar*
generate_message_id (void)
{
	unsigned long now;
	unsigned long randy;
	static unsigned char baseid[64];
	gchar *b, *t, tmp[32];
	char *chars32 = "0123456789abcdefghijklmnopqrstuv";
	static unsigned long last_now;

	while (1) {
		time((time_t *) & now);
		if (now != last_now)
			break;
		sleep (1);
	}
	last_now = now;
	srand (now + getpid());
	randy = (unsigned long) rand();

	b = baseid;
	t = tmp;
	while (now) {
		*t++ = chars32[now & 0x1F];
		now = now >> 5;
	}

	while (t > tmp) {
		t--;
		*b++ = *t;
	}
	*b++ = '.';

	t = tmp;
	while (randy) {
		*t++ = chars32[randy & 0x1F];
		randy = randy >> 5;
	}
	while (t > tmp) {
		t--;
		*b++ = *t;
	}

	*b++ = '@';
	return (char *) baseid;
}


/*--------------------------------------------------------------------
 * FIXME: move into util.c (or make me static?)
 *--------------------------------------------------------------------*/
static gchar *
get_host_name (void)
{
	gchar *ptr;
	gchar hostname[MAXHOSTNAMELEN + 1];

	hostname[0] = '\0';

	if ((gethostname(hostname, sizeof(hostname))) == -1) {
		if ((ptr = getenv("HOST")) != NULL)
			strncpy(hostname, ptr, MAXHOSTNAMELEN);
		else if ((ptr = getenv("HOSTNAME")) != NULL)
			strncpy(hostname, ptr, MAXHOSTNAMELEN);
		else
			hostname[0] = '\0';
	}
	hostname[MAXHOSTNAMELEN] = '\0';
	ptr = strtok (hostname, ".");
	return g_strdup ( ptr );
}


/*--------------------------------------------------------------------
 * get the fully qualified domain name
 *--------------------------------------------------------------------*/
static gchar *
get_fqdn (gchar *host)
{
	char name[MAXHOSTNAMELEN + 2];
	char line[1025];
	char *cp, *domain;
	static char fqdn[1024];
	struct hostent *hp;
	struct in_addr in;
	FILE *inf;

	*fqdn = '\0';
	domain = NULL;

	name[MAXHOSTNAMELEN] = '\0';
	if (host) {
		if (strchr(host, '.'))
			return host;
		strncpy(name, host, MAXHOSTNAMELEN);
	}
	else {
		if (gethostname(name, MAXHOSTNAMELEN))
			return NULL;
	}

	if ('0' <= *name && *name <= '9') {
		in.s_addr = inet_addr(name);
		if ((hp = gethostbyaddr((char *) &in.s_addr, 4, AF_INET)))
			in.s_addr = (*hp->h_addr);
		return (hp && strchr(hp->h_name, '.') ? hp->h_name : (gchar *) inet_ntoa(in));
	}
	if ((hp = gethostbyname(name)) && !strchr(hp->h_name, '.'))
		if ((hp = gethostbyaddr(hp->h_addr, hp->h_length, hp->h_addrtype)))
			in.s_addr = (*hp->h_addr);

	sprintf(fqdn, "%s", hp ? strchr(hp->h_name, '.')
		? hp->h_name : (gchar *) inet_ntoa(in)
		: "");
	if (!*fqdn || (fqdn[strlen(fqdn) - 1] <= '9')) {
		*fqdn = '\0';
		inf = fopen("/etc/resolv.conf", "r");
		if (inf) {
			while (fgets(line, 1024, inf)) {
				line[1024] = '\0';
				(void) Pan_strtrim(line);
				if (strncmp(line, "domain ", 7) == 0) {
					domain = line + 7;
					break;
				}
				if (strncmp(line, "search ", 7) == 0) {
					domain = line + 7;
					cp = strchr(domain, ' ');
					if (cp)
						*cp = '\0';
					break;
				}
			}
			if (domain)
				sprintf(fqdn, "%s.%s", name, domain);
			fclose(inf);
		}
	}
	return (fqdn);
}

static gchar *
reformat_body (gchar *body)
{
	gchar *tmp;
	gchar *body2 = g_malloc0 (sizeof (body));

	while ((tmp = strtok (body, "\n"))) {
		tmp = wrap_long_lines (tmp, 78);
		g_strconcat (body2, tmp, NULL);
	}

	return body2;
}

void
message_send (GtkWidget *widget, Message *msg)
{
	gchar *subject, *from, *newsgroups, *message, *path, *organization;
	gchar *tmp;
	gchar *buf;
	static char *host = NULL;
	static char *uname = NULL;
	static char *fqdn = NULL;
	gchar *body;
	gchar *err;

	if (msg->window == NULL)
		g_error ("WTF!! msg->window == NULL\n");
	if (host == NULL)
		host = get_host_name ();
	if (fqdn == NULL)
		fqdn = get_fqdn (host);
	if (uname == NULL)
		uname = g_get_user_name ();

	path = g_strconcat (host, "!", uname, NULL);
	g_free ( host );

	from = gtk_editable_get_chars (GTK_EDITABLE (msg->window->from), 0, -1);
	newsgroups = gtk_editable_get_chars (GTK_EDITABLE (msg->window->newsgroups), 0, -1);
	subject = gtk_editable_get_chars (GTK_EDITABLE (msg->window->subject), 0, -1);
	organization = gtk_entry_get_text (GTK_ENTRY(msg->window->organization));

	body = gtk_editable_get_chars ( GTK_EDITABLE(msg->window->body), 0, -1);
	err = check_body ( body );
	if ( err != NULL ) {
		pan_error_dialog ( _("Unable to post message:\n%s"), err );
		g_free ( body );
		return;
	}
	g_free ( body );

/* FIXME: doesn't do anything
	if (!check_from_syntax(from))
		g_error ("Bad From. Do something!");
	if (!check_newsgroups_syntax(newsgroups))
		g_error ("Bad Newsgroups. Do something!");
*/
	message = g_strconcat ("From: ", from, "\n",
			       "Path: ", path, "\n",
			       "Newsgroups: ", newsgroups, "\n",
			       "Subject: ", subject, "\n", NULL);
	g_free (path);
	g_free (from);
	g_free (newsgroups);
	g_free (subject);
	tmp = message;
	
	if (Pan.active_server->gen_msgid) {
		message = g_strconcat (tmp,"Message-ID: <", generate_message_id(), fqdn, ">\n", NULL);
		g_free (tmp);
		tmp = message;
	}
	if (msg) {
		gchar* pch = generate_references ( msg );
		message = g_strconcat (message, "References: ", pch, "\n", NULL );
		g_free ( pch );
		g_free ( tmp );
		tmp = message;
	}
	body = g_strconcat ("\n", gtk_editable_get_chars (GTK_EDITABLE(msg->window->body), 0, -1), NULL);
	buf = generate_date ();
	message = g_strconcat (tmp,
			       "Date: ", buf, "\n",
			       "Organization: ", organization, "\n",
			       "X-Newsreader: ", "Pan ", VERSION, "\n",
			       wrap_long_lines (body, 78), NULL);
	g_free (buf);
	g_free (tmp);
	g_free (body);

	queue_add_post ( message );
	g_free ( message );

	gtk_widget_destroy ( msg->window->window );
	message_free ( msg );
}


static GSList *
parse_to_field (MessageWindow *mw)
{
	return NULL;
}


/*--------------------------------------------------------------------
 *
 *--------------------------------------------------------------------*/
static gint
determine_transport_type (const gchar *address)
{
	if ( strcasecmp (address, "@"))
		return EMAIL;
	else if ( strcasecmp (address, "."))
		return NNTP;
	else
		return 0;
}
